# This file is part of Build Your Own Virtual machine.
#
# Copyright 2018 Vincent Ladeuil.
# Copyright 2016, 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
import unittest

from byot import scenarii
from byov import config
from byov.tests import (
    features,
    fixtures,
)

load_tests = scenarii.load_tests_with_scenarios


# FIXME: parametrized by distribution -- vila 2017-12-07
class TestStatusPerVm(unittest.TestCase):

    scenarios = [(k, dict(kls=k)) for k in config.vm_class_registry.keys()]

    def setUp(self):
        super(TestStatusPerVm, self).setUp()
        fixtures.setup_tests_config(self)
        # Some classes require additional features
        required_features = {
            'ephemeral-lxd': features.lxd_client_feature,
            'libvirt': features.use_sudo_for_tests_feature,
            'lxd': features.lxd_client_feature,
            'nova': features.nova_creds,
            'scaleway': features.scaleway_creds,
        }
        if self.kls in required_features:
            f = required_features[self.kls]
            if not f.available():
                self.skipTest('{} is not available'.format(f.feature_name()))
        self.conf = config.VmStack('foo')
        self.conf.set('vm.name', 'foo')
        self.conf.set('vm.class', self.kls)
        self.conf.set('vm.release', 'xenial')
        self.conf.set('vm.architecture', 'amd64')

    def test_unknown_vm(self):
        vm_class = self.conf.get('vm.class')
        vm = vm_class(self.conf)
        self.assertEqual('UNKNOWN', vm.state())
