# This file is part of Build Your Own Virtual machine.
#
# Copyright 2018 Vincent Ladeuil.
# Copyright 2016, 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals

import io
import os
import unittest

from byot import scenarii
from byov import config
from byov.tests import fixtures


load_tests = scenarii.load_tests_with_scenarios


# FIXME: parametrized by distribution -- vila 2017-12-07
class TestUpdate(unittest.TestCase):

    scenarios = [(k, dict(kls=k, series='xenial', arch='amd64'))
                 for k in config.vm_class_registry.keys()
                 # - ephemeral lxcs are not meant to install packages
                 # - something is broken with libvirt (console owned by root is
                 #   back /o\)
                 if k not in ('libvirt', 'ephemeral-lxd',)]

    def setUp(self):
        super(TestUpdate, self).setUp()
        fixtures.setup_tests_config(self)
        fixtures.set_uniq_vm_name(self)
        self.log_stream = io.StringIO()
        fixtures.override_logging(self, self.log_stream)
        fixtures.per_backend_release_arch_setup(self)

    def prepare_vm(self, packages):
        """Prepare a vm to be setup with a list of packages as a string.

        It's assumed that tests will call vm.setup() so we prepare cleanup
        here.
        """
        conf = config.VmStack(self.vm_name)
        vm_class = conf.get('vm.class')
        vm = vm_class(conf)
        vm.conf.set('vm.packages', packages)
        vm.conf.set('vm.update', 'true')
        self.addCleanup(vm.teardown, True)
        return vm

    def test_usable_vm(self):
        vm = self.prepare_vm('')
        # There is more than meet the eye in the following test (a single call
        # to setup()). What is really tested here is that from any image, a
        # full dist-upgrade can be performed successfully. So depending on the
        # image freshness, it is expected that the test fail and help writing
        # more focused tests depending on the encountered errors.
        vm.setup()

    def test_setup_scripts(self):
        vm = self.prepare_vm('')
        vm.conf.set('vm.setup_scripts', 'testme')
        with open('testme', 'w')as f:
            f.write('#!/bin/sh\ntouch testme-ran\n')
        os.chmod('testme', 0o755)
        vm.setup()
        ret, out, err = vm.shell_captured('ls', 'testme*ran')
        self.assertEqual(0, ret)
        self.assertEqual('testme-ran\n', out)
        self.assertEqual('', err)

# MISSINGTESTS: errors in update_sources and upgrade_packages.
